#/////////////////////////////////////////////////////////////////////////////
#
# Program : AutoReplace (for aMSN) v1.0 /20061228
#---------------------------------------------
# Author : "Mathieu Papineau" <mpa@hammock.fr>
#
#/////////////////////////////////////////////////////////////////////////////

namespace eval ::AutoReplace {
	variable config
	variable configlist
	variable replacements
	variable ARdir

	#/////////////////////////////////////////////////////////////////////
	# ::AutoReplace::Init
	#--------------------------------------
	# register events and initialize
	#/////////////////////////////////////////////////////////////////////
	proc Init {dir} {
		::plugins::RegisterPlugin "AutoReplace"
		::plugins::RegisterEvent "AutoReplace" chat_msg_send DoReplacementOnReturn
		::plugins::RegisterEvent "AutoReplace" chatwindowbutton BindForTypingMode
		::plugins::RegisterEvent "AutoReplace" PluginConfigured SaveReplacements
		set ::AutoReplace::ARdir $dir

		array set ::AutoReplace::config {
			enabled {1}
			enabled_while_typing {1}
		}

		set ::AutoReplace::configlist [list \
			[list bool "Enabled" enabled] \
			[list bool "Enabled while typing" enabled_while_typing] \
			[list frame ::AutoReplace::BuildFrameConf ""] \
		]

		LoadReplacements $dir
		set ::AutoReplace::sendMsg 0
	}

	#/////////////////////////////////////////////////////////////////////
	# ::AutoReplace::BindForTypingMode
	#-------------------------------------------------------
	# link the keypress event to the replacement proc
	#/////////////////////////////////////////////////////////////////////
	proc BindForTypingMode {event evpar} {
		upvar 2 $evpar newvar
		set inputbox [::ChatWindow::GetInputText $newvar(window_name)]
		bind $inputbox <space> "after 0 ::AutoReplace::DoReplacementWhileTyping $newvar(window_name)"
	}

	#/////////////////////////////////////////////////////////////////////
	# ::AutoReplace::BuildFrameConf
	#/////////////////////////////////////////////////////////////////////
	proc BuildFrameConf {wn} {
		frame $wn.ar -padx 5 -pady 5
		frame $wn.ar.g -padx 2 -pady 2
		label $wn.ar.g.lblreps -text "REPLACEMENTS"
		frame $wn.ar.g.f1 -borderwidth 2 -padx 5 -pady 5
		button $wn.ar.g.f1.btadd -text "Add" -command "::AutoReplace::FC_btadd_click $wn.ar"
		button $wn.ar.g.f1.btdel -text "Delete" -command "::AutoReplace::FC_btdel_click $wn.ar"
		button $wn.ar.g.f1.btmod -text "Modify" -command "::AutoReplace::FC_btmod_click $wn.ar"
		frame $wn.ar.g.f2 -relief ridge -borderwidth 2 -padx 5 -pady 5
		label $wn.ar.g.f2.lblpat -text "Pattern :"
		label $wn.ar.g.f2.lblsub -text "Substitution :"
		entry $wn.ar.g.f2.epat
		entry $wn.ar.g.f2.esub
		frame $wn.ar.g.f3 -borderwidth 2 -padx 5 -pady 5
		button $wn.ar.g.f3.btreload -text "Reload" -command "::AutoReplace::LoadReplacements {$::AutoReplace::ARdir}"
		button $wn.ar.g.f3.btup -text "Up" -command "::AutoReplace::FC_btup_click $wn.ar"
		button $wn.ar.g.f3.btdown -text "Down" -command "::AutoReplace::FC_btdown_click $wn.ar"
		frame $wn.ar.d
		listbox $wn.ar.d.lbreps -height 11 -selectmode single -listvariable ::AutoReplace::replacements -exportselection 0
		scrollbar $wn.ar.d.sblbreps -orient vertical -height 192
		pack $wn.ar.g.f1.btadd -side left -padx 5
		pack $wn.ar.g.f1.btdel -side left -padx 5
		pack $wn.ar.g.f1.btmod -side left -padx 5
		pack $wn.ar.g.f2.esub -side bottom
		pack $wn.ar.g.f2.lblsub -side bottom -anchor w
		pack $wn.ar.g.f2.epat -side bottom
		pack $wn.ar.g.f2.lblpat -side bottom -anchor w
		pack $wn.ar.g.f3.btreload -side left -padx 5
		pack $wn.ar.g.f3.btup -side left -padx 5
		pack $wn.ar.g.f3.btdown -side left -padx 5
		pack $wn.ar.d.lbreps -side left
		pack $wn.ar.d.sblbreps -side right
		pack $wn.ar.g.lblreps -side top
		pack $wn.ar.g.f3 -side bottom
		pack $wn.ar.g.f1 -side bottom
		pack $wn.ar.g.f2
		pack $wn.ar.g -side left -anchor n
		pack $wn.ar.d -side right
		pack $wn.ar
		$wn.ar.d.lbreps configure -yscrollcommand [list $wn.ar.d.sblbreps set]
		$wn.ar.d.sblbreps configure -command [list $wn.ar.d.lbreps yview]
		bind $wn.ar.d.lbreps <<ListboxSelect>> "::AutoReplace::FC_lbreps_select $wn.ar"
}

	#/////////////////////////////////////////////////////////////////////
	# FrameConf procedures
	#/////////////////////////////////////////////////////////////////////
	proc FC_lbreps_select {frm} {
		set cursel [$frm.d.lbreps curselection]
		if {[llength $cursel] > 0} {
			$frm.g.f2.epat delete 0 end
			$frm.g.f2.epat insert 0 [lindex [lindex $::AutoReplace::replacements $cursel] 0]
			$frm.g.f2.esub delete 0 end
			$frm.g.f2.esub insert 0 [lindex [lindex $::AutoReplace::replacements $cursel] 1]
		}
	}

	proc FC_btadd_click {frm} {
		$frm.d.lbreps insert end [list [$frm.g.f2.epat get] [$frm.g.f2.esub get]]
	}

	proc FC_btmod_click {frm} {
		set cursel [$frm.d.lbreps curselection]
		if {[llength $cursel] > 0} {
			set ::AutoReplace::replacements [lreplace $::AutoReplace::replacements $cursel $cursel [list [$frm.g.f2.epat get] [$frm.g.f2.esub get]]]
		}
	}

	proc FC_btdel_click {frm} {
		set cursel [$frm.d.lbreps curselection]
		if {[llength $cursel] > 0} {
			$frm.d.lbreps delete $cursel
		}
	}

	proc FC_btup_click {frm} {
		set cursel [$frm.d.lbreps curselection]
		if {$cursel > 0} {
			set itemp [lindex $::AutoReplace::replacements [expr $cursel - 1]]
			set lstmp [lreplace $::AutoReplace::replacements [expr $cursel - 1] [expr $cursel - 1] [lindex $::AutoReplace::replacements $cursel]]
			set ::AutoReplace::replacements [lreplace  $lstmp $cursel $cursel $itemp]
			$frm.d.lbreps selection clear 0 end
			$frm.d.lbreps selection set [expr $cursel - 1]
		}
	}

	proc FC_btdown_click {frm} {
		set cursel [$frm.d.lbreps curselection]
		if {$cursel < [expr [llength $::AutoReplace::replacements] - 1]} {
			set itemp [lindex $::AutoReplace::replacements [expr $cursel + 1]]
			set lstmp [lreplace $::AutoReplace::replacements [expr $cursel + 1] [expr $cursel + 1] [lindex $::AutoReplace::replacements $cursel]]
			set ::AutoReplace::replacements [lreplace  $lstmp $cursel $cursel $itemp]
			$frm.d.lbreps selection clear 0 end
			$frm.d.lbreps selection set [expr $cursel + 1]
		}
	}

	#/////////////////////////////////////////////////////////////////////
	# ::AutoReplace::ApplyReplacements
	#/////////////////////////////////////////////////////////////////////
	proc ApplyReplacements {txt} {
		foreach r $::AutoReplace::replacements {
			foreach {pat sub} $r {
				regsub -all "$pat" "$txt" "$sub" txt
			}
		}
		return $txt
	}

	#/////////////////////////////////////////////////////////////////////
	# ::AutoReplace::DoReplacement
	#-------------------------------------------------------
	# NOTE: repositionning the insert cursor won't work with >1.line text
	#/////////////////////////////////////////////////////////////////////
	proc DoReplacementWhileTyping {win_name} {
		if {$::AutoReplace::config(enabled_while_typing)} { 
			set inputbox [::ChatWindow::GetInputText $win_name]
			set txt [$inputbox get 1.0 end-1c]
			set orilen [string length $txt]
			set curpos [split [$inputbox index insert] "."]
			set ypos [lindex $curpos 0]
			set xpos [lindex $curpos 1]
			$inputbox delete 1.0 end
			set txt [ApplyReplacements $txt]
			set diflen [expr [string length $txt] - $orilen]
			$inputbox insert 1.0 $txt
			$inputbox mark set insert "$ypos.[expr $xpos + $diflen]"
		}
	}

	#/////////////////////////////////////////////////////////////////////
	# ::AutoReplace::DoReplacementOnReturn
	#/////////////////////////////////////////////////////////////////////
	proc DoReplacementOnReturn {event evpar} {
		upvar 2 msg msg
		if {$::AutoReplace::config(enabled)} {
			set msg [ApplyReplacements $msg]
		}
	}

	#/////////////////////////////////////////////////////////////////////
	# ::AutoReplace::LoadReplacements
	#/////////////////////////////////////////////////////////////////////
	proc LoadReplacements {dir} {
		set ::AutoReplace::replacements [list]
		set repfile [open "[file join $dir replacements.txt]" "r"]
		set rawreps [read -nonewline $repfile]
		close $repfile
		set reps [split $rawreps "\n"]
		foreach rep $reps {
			lappend ::AutoReplace::replacements [split $rep "	"]
		}
	}

	#/////////////////////////////////////////////////////////////////////
	# ::AutoReplace::SaveReplacements
	#/////////////////////////////////////////////////////////////////////
	proc SaveReplacements {event evpar} {
 		upvar 2 $evpar newvar
 		if {$newvar(name)=="AutoReplace"} {
			set new [file join $::AutoReplace::ARdir "replacements.txt.new"]
			set cur [file join $::AutoReplace::ARdir "replacements.txt"]
			set old [file join $::AutoReplace::ARdir "replacements.txt.bak"]

			set repfile [open "$new" "w"]
			foreach reps $::AutoReplace::replacements {
				puts $repfile "[lindex $reps 0]	[lindex $reps 1]"
			}
			close $repfile
			file rename -force "$cur" "$old"
			file rename "$new" "$cur"
 		}
	}
}

